package at.ac.tuwien.dbai.staff.dvorak.alternation.examples;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
/**
 * A boolean circuit
 * @author Wolfgang Dvorak
 *
 */
public class Circuit{
	/**
	 * The different types of gates
	 * @author Wolfgang Dvorak
	 *
	 */
	enum GateTypes {AND,OR,NOT,INPUT,OUTPUT}; //Output -> OR
	/**
	 * A Gate in the boolean circiut
	 * 
	 * @author Wolfgang Dvorak
	 *
	 * @param <GateTypes> Type of the Gates
	 */
	class Gate<GateTypes> {
		Set<Gate<GateTypes>> In;
		Set<Gate<GateTypes>> Out;
		public GateTypes type;

		public Gate(GateTypes type) {
			this.In=new HashSet<Gate<GateTypes>>();
			this.Out=new HashSet<Gate<GateTypes>>();
			this.type=type;
		}

		boolean addOut(Gate<GateTypes> to){
			return this.Out.add(to);
		};

		boolean addIn(Gate<GateTypes> from){
			return this.In.add(from);
		};

		public Set<Gate<GateTypes>> getIn() {
			return In;
		}

		public Set<Gate<GateTypes>> getOut() {
			return Out;
		}
	}

	Map<Integer,Gate<GateTypes>> gates = new HashMap<Integer,Gate<GateTypes>>();
	List<Gate<GateTypes>> inputGates = new  ArrayList<Gate<GateTypes>>();
	List<Gate<GateTypes>> outputGates= new ArrayList<Gate<GateTypes>>();

	/**
	 * add a gate to the circuit
	 * @param gateNumber gate identifier
	 * @param type  type of the gate
	 */
	public void addGate(int gateNumber,GateTypes type){
		Gate<GateTypes> gate=new Gate<GateTypes>(type);
		gates.put(gateNumber, gate);
		switch(type)
		{
			case INPUT: inputGates.add(gate);break;
			case OUTPUT: outputGates.add(gate);break;
		}
	}

	/**
	 * 
	 * @param gateNumber identifier
	 * @return the gate with the specified identifier
	 */
	public Gate<GateTypes> getGate(int gateNumber){
		return gates.get(gateNumber);
	}

	/**
	 * connect a gates output with another gates input
	 * @param from
	 * @param to
	 */
	public void connect (Gate<GateTypes> from,Gate<GateTypes> to){
		from.addOut(to);
		to.addIn(from);
	}

	/**
	 * 
	 * @param index 
	 * @return the specified output Gate
	 */
	public Gate<GateTypes> getOutGate(int index){
		return outputGates.get(index);
	}
	/**
	 * 
	 * @param index
	 * @return the specified input Gate
	 */
	public Gate<GateTypes> getInputGate(int index){
		return inputGates.get(index);
	}

	/**
	 * 
	 * @param gate
	 * @return number in the ordering of input gates
	 */
	public int getInputIndex(Gate<GateTypes> gate){
		for (int i=0;i<inputGates.size();i++){
			if (inputGates.get(i).equals(gate)) return i;
		}
		return -1;
	}
}
