package at.ac.tuwien.dbai.staff.dvorak.alternation.examples;
import java.util.Set;
import java.util.List;
import java.util.Arrays;


public class PEBBLE {
	private at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATM<Inputtape,Worktape> atm;

	public class Inputtape {
		Set<String> pebbles;
		Set<List<String>> rules;
		String goal;
		String[] startSet;

		private Inputtape(Set<String> pebbles,Set<List<String>> rules,String goal,String[] startSet) {
			super();
			this.pebbles=pebbles;
			this.rules=rules;
			this.goal=goal;
			this.startSet=startSet;
		}
	}

    public class Worktape implements at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.InterfaceWorktape<Inputtape> {
    	String[] S;
    	int counter;

    	private Worktape(){
    		super();
    	}

    	private Worktape(Worktape worktape) {
    		this.counter = worktape.counter;
    		this.S = worktape.S.clone();
    	}

    	public void reset(Inputtape atmInputtape) {

    		this.S = atmInputtape.startSet.clone();
    		this.counter = (int)Math.pow(atmInputtape.pebbles.size(),atmInputtape.startSet.length);
    	}

    	public Worktape clone(){
    		return new Worktape(this);
    	};

    	public boolean equals( Object o ) {
        	if ( o == null ) return false;
            if (! o.getClass().equals(getClass()) ) return false;
            Worktape tape2=(Worktape) o;
    		if(this.counter!=tape2.counter) return false;
    		if(this.S==null){
    			if(!(tape2.S==null)) return false;
    		} else {
    		    if(!java.util.Arrays.equals(this.S,tape2.S)) return false;
    		}
            return true;
    	}

    	@Override
    	public int hashCode() {
    		final int prime = 31;
    		int result = 1;


    		result = prime * result +  ((Object)counter).hashCode();
    		result =  prime * result + java.util.Arrays.hashCode(S);

    		return result;
    	}

    	public String toString() {
    		StringBuffer ausgabe= new StringBuffer();
    		 ausgabe.append("counter=(" +((Object)counter).toString()+"); ");
    		return ausgabe.toString();
    	}
    }

    private class player1 extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		Arrays.sort(atmWorktape.S);
    				if (Arrays.binarySearch(atmWorktape.S,atmInputtape.goal)>=0) return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    				if (atmWorktape.counter<0) return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;

    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, false);
    		atmWorktape2=atmWorktape.clone();
    		for(List<String> rule : atmInputtape.rules){
    						if(Arrays.binarySearch(atmWorktape.S,rule.get(0))>=0 && Arrays.binarySearch(atmWorktape.S,rule.get(1))>=0 && Arrays.binarySearch(atmWorktape.S,rule.get(2))<0){
    							{
    								atmWorktape.S[Arrays.binarySearch(atmWorktape.S,rule.get(0))]=rule.get(2);
    								atmWorktape.counter--;
    							}
    		atmResultTuple.add((byte)1,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    					}
    		return atmResultTuple;
    	}
    }

    private class player2 extends at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>  {
    	
    	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> compute(Worktape atmWorktape) {
    		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape> atmResultTuple;
    		Worktape atmWorktape2;
    		Arrays.sort(atmWorktape.S);
    				if (Arrays.binarySearch(atmWorktape.S,atmInputtape.goal)>=0) return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.ACCEPT;
    				if (atmWorktape.counter<0) return at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple.REJECT;
    						
    				atmResultTuple = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ResultTuple<Worktape>(false, false, true);
    		atmWorktape2=atmWorktape.clone();
    		for(List<String> rule : atmInputtape.rules){
    						if(Arrays.binarySearch(atmWorktape.S,rule.get(0))>=0 && Arrays.binarySearch(atmWorktape.S,rule.get(1))>=0 && Arrays.binarySearch(atmWorktape.S,rule.get(2))<0){
    							{
    								atmWorktape.S[Arrays.binarySearch(atmWorktape.S,rule.get(0))]=rule.get(2);
    								atmWorktape.counter--;
    							}
    		atmResultTuple.add((byte)0,atmWorktape);
    		atmWorktape=atmWorktape2.clone();
    						}
    					}
    		return atmResultTuple;
    	}
    }

    public boolean compute(Set<String> pebbles,Set<List<String>> rules,String goal,String[] startSet) {

    	Inputtape atmInput = new Inputtape(pebbles,rules,goal,startSet);
    	if (this.atm==null) {
    		Worktape atmWork = new  Worktape();
    		java.util.Map<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>> atmStates = new java.util.HashMap<Byte,at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.State<Inputtape,Worktape>>();

    		atmStates.put((byte)0,new player1());
    		atmStates.put((byte)1,new player2());

    		this.atm = new at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ATMacyclic<Inputtape,Worktape>(atmStates, atmWork, (byte)0);
        }
        return atm.compute(atmInput);
    }

	public at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> getComputationTree(){
		at.ac.tuwien.dbai.staff.dvorak.alternation.runtime.ComputationTree<Worktape> cT=this.atm.getComputationTree();
		cT.setStateIdToName((byte)0, "player1");
		cT.setNameToStateId("player1",(byte) 0);
		cT.setStateIdToName((byte)1, "player2");
		cT.setNameToStateId("player2",(byte) 1);

		return cT;
	}
}