package at.ac.tuwien.dbai.staff.dvorak.alternation.runtime;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
/**
 * Computation Tree of an alternating computation - saves the configurations and their succesor relation
 *
 * @author Wolfgang Dvorak
 *
 * @param <GWorktape>  container class for the work variables
 */
public class ComputationTree<GWorktape extends InterfaceWorktape> extends HashMap<ComputationNode<GWorktape>,Boolean>{

	private Map<Byte,String> stateIdToName=new HashMap<Byte,String>();
	private Map<String,Byte> nameToStateId=new HashMap<String,Byte>();

	private ComputationNode<GWorktape> root;
	protected Map<ComputationNode<GWorktape>,ComputationNode<GWorktape>> nodes= new HashMap<ComputationNode<GWorktape>,ComputationNode<GWorktape>>();
	protected Map<ComputationNode<GWorktape>,List<ComputationNode<GWorktape>>> edges= new HashMap<ComputationNode<GWorktape>,List<ComputationNode<GWorktape>>>();

	private static final long serialVersionUID = 56155764518742618L;

	/**
	 * Adds a edge between two Computation Nodes
	 * @param parent fromNode
	 * @param child toNode
	 */
	public void addEdge(ComputationNode<GWorktape> parent, ComputationNode<GWorktape> child){
		ComputationNode<GWorktape> localParent=parent;
		ComputationNode<GWorktape> localChild=child;
		if(nodes.containsKey(parent)){
			localParent=nodes.get(parent);
		}

		if(nodes.containsKey(child)){
			localChild=nodes.get(child);
		}

		if (!edges.containsKey(localParent)){
			edges.put(localParent, new ArrayList<ComputationNode<GWorktape>>());
		}
		edges.get(localParent).add(localChild);

	}
	/**
	 *
	 * @return the root of the computation tree
	 */
	public ComputationNode<GWorktape> getRoot() {
		return root;
	}

	/**
	 *
	 * @param root ComputationNode constucted by the input and the initial worktape
	 */
	public void setRoot(ComputationNode<GWorktape> root) {
		this.root = root;
	}

	/**
	 *
	 * @param node ComputationNode
	 * @return Successors in the ComputationTree
	 */
	public List<ComputationNode<GWorktape>> getChildren(ComputationNode<GWorktape> node){
		return edges.get(node);
	}

	/**
	 * maps the state id with its name
	 * @param b state id
	 * @param string state name
	 */
	public void setStateIdToName(Byte b, String string) {
		this.stateIdToName.put(b,string);

	}

	/**
	 * maps the state name with its id
	 * @param string state name
	 * @param b state id
	 */
	public void setNameToStateId(String string, Byte b) {
		this.nameToStateId.put(string, b);
	}

	/**
	 * returns a states name for a given id
	 * @param stateId state id
	 * @return state name
	 */
	public String getStateName(Byte stateId){
		return stateIdToName.get(stateId);
	}

	/**
	 * returns a states id for a given name
	 * @param name state name
	 * @return state id
	 */
	public Byte getStateId(String name){
		return nameToStateId.get(name);
	}

	public Boolean put(ComputationNode<GWorktape> node,Boolean acceptance){
		Boolean returnValue=super.put(node,acceptance);
		nodes.put(node,node);
		return returnValue;
	}

}